SUBROUTINE gradpmf(vec,i)
! Systematic force associated to the Free Energy Surface (FES)
! Note: dgauss = 1.0/dgauss**2
!       ctgauss includes number of states for computing the mean value
use ftsmod
use errormod

implicit none
! input/output variables
integer i
real vec(ncv)
! local variables
integer j, k, l
integer ixmax, iymax, izmax, ix, iy, iz
integer nx, ny, nz, ipx, ipy, ipz
real expval, theta(ncv), sint(ncv), biasg(ncv)
real x, y, z, vab1, vab2, fesval
real mx, dmx, my, dmy, mz, dmz, m2
real t, u
real, parameter :: zero=1.0e-15

! ** Transformation from cartesian to angle CVs:
do j = 1, ncv
  theta(j) = zrep(j,i)
  sint(j) = 1.0
  if (Qangcv) then
    if (angcv(j)) then
      theta(j) = acos(zrep(j,i)/dcv(j))
      sint(j) = sin(theta(j))
      if (abs(sint(j)).lt.zero) sint(j) = zero
      sint(j) = -1.0/(dcv(j)*sint(j))
    endif
  endif
enddo
! ** Computation of systematic force:
if (Qgauss) then ! from Gaussian hills file
  do j = 1, ncv
    biasg(j) = 0.0
    do k = 1, nigauss
      expval = 0.0
      do l = 1, ncv
        expval = expval - (theta(l)-zgauss(l,k))**2*dgauss(l)*0.5
      enddo
      biasg(j) = biasg(j) + wgauss(k)*(zgauss(j,k)-theta(j))*exp(expval)
    enddo
  enddo
  do j = 1, ncv
    vec(j) = biasg(j)
  enddo
  do j = 1, ncv
    k = nigauss
    do while (k<ngauss)
      k = k + 1
      expval = 0.0
      do l = 1, ncv
        expval = expval - (theta(l)-zgauss(l,k))**2*dgauss(l)*0.5
      enddo
      biasg(j) = biasg(j) + wgauss(k)*(zgauss(j,k)-theta(j))*exp(expval)
      vec(j) = vec(j) + biasg(j)
    enddo    
    vec(j) = ctgauss*vec(j)*dgauss(j)*sint(j)
  enddo
else if (QBspline) then ! B-spline interpolation
  vec = 0.0
  ixmax = ng(1)
  iymax = ng(2)
  if (theta(1)<xg(1,1) .or. theta(1)>xg(ixmax,1)) call error ('gradpmf', 'Point is out of the CV grid', faterr)
  if (theta(2)<xg(1,2) .or. theta(2)>xg(iymax,2)) call error ('gradpmf', 'Point is out of the CV grid', faterr)
  ! CV coordinates in the local grid system
  x = theta(1) - xg(1,1)
  y = theta(2) - xg(1,2)
  ! integer counter for CV coordinates   
  ix = nint(x*idcel(1))
  iy = nint(y*idcel(2))
  if (ix.eq.0) ix = 1
  if (iy.eq.0) iy = 1
  if (ix.eq.(ixmax-1)) ix = ixmax - 2
  if (iy.eq.(iymax-1)) iy = iymax - 2
  if (ncv==2) then
    ! distribution by 9 adjacent grid points
    do nx = ix-1, ix+1
      ipx = nx*iymax
      vab1 = 1.5 + (nx*dcel(1)-x)*idcel(1)
      vab2 = vab1 - 1.0
      mx = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
      dmx = m2(vab1) - m2(vab2)
      do ny = iy-1, iy+1
        ipy = ny + ipx + 1
        vab1 = 1.5 + (ny*dcel(2)-y)*idcel(2)
        vab2 = vab1 - 1.0
        my = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
        dmy = m2(vab1) - m2(vab2)
        fesval = fesg(ipy)
        vec(1) = vec(1) + dmx*my*fesval*idcel(1)
        vec(2) = vec(2) + mx*dmy*fesval*idcel(2)   
      enddo
    enddo
    vec(1) = sint(1)*vec(1)
    vec(2) = sint(2)*vec(2)
  else ! ncv=3
    izmax = ng(3)
    if (theta(3)<xg(1,3) .or. theta(3)>xg(izmax,3)) call error ('gradpmf', 'Point is out of the CV grid', faterr)
    ! CV coordinates in the local grid system
    z = theta(3) - xg(1,3)
    ! integer counter for CV coordinates   
    iz = nint(z*idcel(3))
    if (iz.eq.0) iz = 1
    if (iz.eq.(izmax-1)) iz = izmax - 2
    ! distribution by 27 adjacent grid points
    do nx = ix-1, ix+1
      ipx = nx*iymax*izmax
      vab1 = 1.5 + (nx*dcel(1)-x)*idcel(1)
      vab2 = vab1 - 1.0
      mx = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
      dmx = m2(vab1) - m2(vab2)
      do ny = iy-1, iy+1
        ipy = ny*izmax + ipx
        vab1 = 1.5 + (ny*dcel(2)-y)*idcel(2)
        vab2 = vab1 - 1.0
        my = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
        dmy = m2(vab1) - m2(vab2)
        do nz = iz-1, iz+1
          ipz = nz + ipy + 1
          vab1 = 1.5 + (nz*dcel(3)-z)*idcel(3)
          vab2 = vab1 - 1.0
          mz = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
          dmz = m2(vab1) - m2(vab2)
          fesval = fesg(ipz) 
          vec(1) = vec(1) + dmx*my*mz*fesval*idcel(1)
          vec(2) = vec(2) + mx*dmy*mz*fesval*idcel(2)
          vec(3) = vec(3) + mx*my*dmz*fesval*idcel(3)
        enddo
      enddo
    enddo
    vec(1) = sint(1)*vec(1)
    vec(2) = sint(2)*vec(2)
    vec(3) = sint(3)*vec(3)
  endif
else if (Qbicubic) then ! bicubic interpolation
  if (theta(1)<xg(1,1) .or. theta(1)>xg(ng(1),1)) call error ('gradpmf', 'Point is out of the CV grid', faterr)
  if (theta(2)<xg(1,2) .or. theta(2)>xg(ng(2),2)) call error ('gradpmf', 'Point is out of the CV grid', faterr)
  ! CV coordinates in the local grid system
  x = theta(1) - xg(1,1)
  y = theta(2) - xg(1,2)
  ! integer counter for CV coordinates and coefficients for the bicubic interpolation
  j = int(x*idcel(1)) + 1
  k = int(y*idcel(2)) + 1
  l = (ng(1)-1)*(k-1) + j 
  ! interpolated gradient values
  t = (theta(1)-xg(j,1))*idcel(1)
  u = (theta(2)-xg(k,2))*idcel(2)
  vec = 0.0
  do nx = 4, 1, -1
    vec(1) = u*vec(1) + (3.0*cg(4,nx,l)*t+2.0*cg(3,nx,l))*t + cg(2,nx,l)
    vec(2) = t*vec(2) + (3.0*cg(nx,4,l)*u+2.0*cg(nx,3,l))*u + cg(nx,2,l)
  enddo
  vec(1) = sint(1)*idcel(1)*vec(1)
  vec(2) = sint(2)*idcel(2)*vec(2)
endif

return
END SUBROUTINE gradpmf
