SUBROUTINE jacobi(a)
! NUMERICAL RECIPES IN FORTRAN 77
! Computes all eigenvalues and eigenvectors of a real symmetric matrix a, which is of size ncv by ncv.
! On output, elements of a above the diagonal are destroyed. lamb returns the eigenvalues of a. Um is 
! a matrix with the same dimensions as a, whose columns contain, on output, the normalized eigenvectors of a. 
! nrot returns the number of Jacobi rotations that were required.
use ftsmod
use errormod
use stdiomod

implicit none
! input/output variables
real a(ncv,ncv)
! local variables
integer i, ip, iq, nrot
real b(ncv), z(ncv), sm, tresh, g, h, t, theta, c, s, tau
character :: wrtline*2048

! Initialize Um to the identity matrix
Um = 0.0
do ip = 1, ncv
  Um(ip,ip) = 1.0
enddo
! Initialize b and lamb to the diagonal of a
do ip = 1, ncv
  b(ip) = a(ip,ip)
  lamb(ip) = b(ip)
enddo
z(:) = 0.0 ! This vector will accumulate terms of the form ta_pq as in eq. (11.1.14) 
nrot = 0
do i = 1, 50
  ! Sum off-diagonal elements
  sm = 0.0
  do ip = 1, ncv-1
    do iq = ip+1, ncv
      sm = sm + abs(a(ip,iq)) 
    enddo
  enddo 
  if (sm==0.0) then ! The normal return, which relies on quadratic convergence to machine underflow
    write(wrtline,*) 'Number of Jacobi rotations = ',nrot
    write(outu,'(6x,a)') trim(adjustl(wrtline)) 
    return
  endif
  tresh = merge(0.2*sm/ncv**2,0.0,i<4) ! On the first three sweeps, we will rotate only if tresh exceeded
  do ip = 1, ncv-1
    do iq = ip+1, ncv
      g = 100.0*abs(a(ip,iq))
      ! After four sweeps, skip the rotation if the off-diagonal element is small
      if ((i>4) .and. (abs(lamb(ip))+g==abs(lamb(ip))) .and. (abs(lamb(iq))+g==abs(lamb(iq)))) then
        a(ip,iq) = 0.0
      else if (abs(a(ip,iq))>tresh) then
        h = lamb(iq) - lamb(ip)
        if (abs(h)+g==abs(h)) then
          t = a(ip,iq)/h
        else
          theta = 0.5*h/a(ip,iq)
          t = 1.0/(abs(theta)+sqrt(1.0+theta**2))
          if (theta<0.0) t = -t
        endif
        c = 1.0/sqrt(1+t**2)
        s = t*c
        tau = s/(1.0+c)
        h = t*a(ip,iq)
        z(ip) = z(ip) - h
        z(iq) = z(iq) + h
        lamb(ip) = lamb(ip) - h
        lamb(iq) = lamb(iq) + h
        a(ip,iq) = 0.0
        call jrotate(a(1:ip-1,ip),a(1:ip-1,iq))
        ! Case of rotations 1 <= j < p
        call jrotate(a(ip,ip+1:iq-1),a(ip+1:iq-1,iq))
        ! Case of rotations p < j < q
        call jrotate(a(ip,iq+1:ncv),a(iq,iq+1:ncv))
        ! Case of rotations q < j <= n
        call jrotate(Um(:,ip),Um(:,iq))
        nrot = nrot + 1
      endif
    enddo
  enddo
  ! Update lamb and reinitialize z
  b(:) = b(:) + z(:)
  lamb(:) = b(:)
  z(:) = 0.0
enddo
call error ('jacobi', 'too many iterations in jacobi', faterr)
CONTAINS

SUBROUTINE jrotate(a1,a2)
real, dimension(:), intent(inout) :: a1,a2
real, dimension(size(a1)) :: wk1

wk1(:) = a1(:)
a1(:) = a1(:) - s*(a2(:)+a1(:)*tau)
a2(:) = a2(:) + s*(wk1(:)-a2(:)*tau)

END SUBROUTINE jrotate

END SUBROUTINE jacobi
